// NCPBUG Charlotte PowerBuilder Conference 2015
// C# Integration Framework
using System;
using System.Text;  
using System.Windows.Forms;            // Control
using Microsoft.Win32;                 // Registeration Needs
using System.Reflection;               // Assembly
using System.Runtime.InteropServices;  // Guid and InterfaceType

namespace NCPBUG
{
    // Declare the GUID and the Interface for Events
    [Guid("11FF02DA-C7F0-4EBA-9DA8-36371BF04B39")]
    [InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]

    // Declare the Events 
    public interface IActiveXEvents
    {
        // DispID specifies the COM dispatch identifier for the event 
        [DispId(1)]
        void CustomEvent();
    }

    // Declare delegates for the Events
    [ComVisible(false)]
    public delegate void delCustomEvent();
    
    // Declare the GUID and the Interface for Methods
    [Guid("BEA75EFD-39CB-46C1-9396-FCD683F34232")]
    [InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]

    // Declare the Methods
    public interface IActiveXPropertiesAndMethods
    {
        [DispId(1)]
        void RaiseCustomEvent();
    }

    // Declare the GUID, Interface and Program ID for the Class
    [Guid("231FA2AA-7EF2-4856-9B5D-E2144AD45496")]
    // ProgIDs present a human-readable version of the class 
    // identifier (CLSID) used to identify COM/ActiveX objects.
    // Format:  name1.name2.version
    [ProgId("NCPNUG.CSFrameWork.01")]
    [ComSourceInterfaces(typeof(IActiveXEvents))]
    [ClassInterface(ClassInterfaceType.None)]

    public partial class CSFrameWork : UserControl, IActiveXPropertiesAndMethods
    {
        // Wire the events using their delegates
        public event delCustomEvent CustomEvent;
    
        // Constructor
        public CSFrameWork()
        { InitializeComponent(); }

        // Define the public methods that consume the events 
        public void RaiseCustomEvent()
        {  CustomEvent(); }

        // Register the UserControl        
        [ComRegisterFunction()]
        public static void RegisterClass(string key)
        {
            // Define the Root
            StringBuilder sbRoot = new StringBuilder(key);
            sbRoot.Replace(@"HKEY_CLASSES_ROOT\", "");

            // Write the Root and create a subclass
            RegistryKey keyReg = Registry.ClassesRoot.OpenSubKey(sbRoot.ToString(), true);
            RegistryKey ctrl = keyReg.CreateSubKey("Control");
            ctrl.Close();

            // Register the codebase
            RegistryKey inprocServer32 = keyReg.OpenSubKey("InprocServer32", true);
            inprocServer32.SetValue("CodeBase", Assembly.GetExecutingAssembly().CodeBase);
            inprocServer32.Close();

            // Regsiter the TypeLibrary; The Guid comes from the application assembly property
            // BIG WARNING - Watch out for missing curly braces or unexpected results will happen!
            string TypeLibGUID = "{c6d8bb8b-e299-4a22-afc4-4825bec727f8}";
            RegistryKey typelib = keyReg.CreateSubKey("TypeLib");
            typelib.SetValue(null, TypeLibGUID);
            typelib.Close();

            keyReg.Close();
        }

        // Un-Register User Control
        [ComUnregisterFunction()]
        public static void UnregisterClass(string key)
        {
            // Define a null root
            StringBuilder sbNullRoot = new StringBuilder(key);
            sbNullRoot.Replace(@"HKEY_CLASSES_ROOT\", "");

            // Get the registry entry and blow away the subkeys
            RegistryKey keyReg = Registry.ClassesRoot.OpenSubKey(sbNullRoot.ToString(), true);
            keyReg.DeleteSubKey("Control", false);

            RegistryKey inprocServer32 = keyReg.OpenSubKey("InprocServer32", true);
            keyReg.DeleteSubKey("CodeBase", false);

            keyReg.Close();
        }
    } 
} 
